@@ -1,5 +1,5 @@
 /*
- * Zapojit - GLib/GObject wrapper for Skydrive and Hotmail
+ * Zapojit - GLib/GObject wrapper for the SkyDrive and Hotmail REST APIs
  * Copyright © 2012 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
@@ -135,6 +135,27 @@ zpj_skydrive_download_file_got_chunk (SoupMessage *message, SoupBuffer *chunk, g
 }
 
 
+static void
+zpj_skydrive_download_file_id_to_stream_in_thread_func (GSimpleAsyncResult *simple,
+                                                        GObject *object,
+                                                        GCancellable *cancellable)
+{
+  ZpjSkydrive *self = ZPJ_SKYDRIVE (object);
+  GError *error;
+  GInputStream *stream;
+  ZpjSkydriveThreadData *data;
+
+  data = (ZpjSkydriveThreadData *) g_simple_async_result_get_op_res_gpointer (simple);
+
+  error = NULL;
+  stream = zpj_skydrive_download_file_id_to_stream (self, data->entry_id, cancellable, &error);
+  if (error != NULL)
+    g_simple_async_result_take_error (simple, error);
+
+  g_value_take_object (&data->result, stream);
+}
+
+
 static void
 zpj_skydrive_list_folder_id_in_thread_func (GSimpleAsyncResult *simple, GObject *object, GCancellable *cancellable)
 {
@@ -415,7 +436,9 @@ zpj_skydrive_delete_entry_id (ZpjSkydrive *self, const gchar *entry_id, GCancell
  * Synchronously returns a stream for downloading the file
  * corresponding to @file_id from
  * <ulink url="http://msdn.microsoft.com/en-us/library/live/hh826521">
- * Skydrive</ulink>.
+ * Skydrive</ulink>. See
+ * zpj_skydrive_download_file_id_to_stream_async() for the asynchronous
+ * version of this call.
  *
  * Returns: (transfer full): A #GInputStream to read the file data
  * from. Free the returned object with g_object_unref().
@@ -470,6 +493,94 @@ zpj_skydrive_download_file_id_to_stream (ZpjSkydrive *self,
 }
 
 
+/**
+ * zpj_skydrive_download_file_id_to_stream_async:
+ * @self: A #ZpjSkydrive.
+ * @file_id: The ID of the #ZpjSkydriveFile to be downloaded.
+ * @cancellable: (allow-none): An optional #GCancellable object, or
+ *   %NULL.
+ * @callback: (scope async): A #GAsyncReadyCallback to call when the
+ *   request is satisfied.
+ * @user_data: (closure): The data to pass to @callback.
+ *
+ * Asynchronously returns a stream for downloading the file
+ * corresponding to @file_id from
+ * <ulink url="http://msdn.microsoft.com/en-us/library/live/hh826521">
+ * Skydrive</ulink>. See zpj_skydrive_download_file_id_to_stream() for
+ * the synchronous version of this call.
+ *
+ * When the operation is finished, @callback will be called. You can
+ * then call zpj_skydrive_download_file_id_to_stream_finish() to get
+ * the result of the operation.
+ */
+void
+zpj_skydrive_download_file_id_to_stream_async (ZpjSkydrive *self,
+                                               const gchar *file_id,
+                                               GCancellable *cancellable,
+                                               GAsyncReadyCallback callback,
+                                               gpointer user_data)
+{
+  GSimpleAsyncResult *simple;
+  ZpjSkydriveThreadData *data;
+
+  g_return_if_fail (ZPJ_IS_SKYDRIVE (self));
+  g_return_if_fail (file_id != NULL && file_id[0] != '\0');
+
+  simple = g_simple_async_result_new (G_OBJECT (self),
+                                      callback,
+                                      user_data,
+                                      zpj_skydrive_download_file_id_to_stream_async);
+  g_simple_async_result_set_check_cancellable (simple, cancellable);
+
+  data = zpj_skydrive_thread_data_new (G_TYPE_INPUT_STREAM, file_id, NULL);
+  g_simple_async_result_set_op_res_gpointer (simple, data, (GDestroyNotify) zpj_skydrive_thread_data_free);
+
+  g_simple_async_result_run_in_thread (simple,
+                                       zpj_skydrive_download_file_id_to_stream_in_thread_func,
+                                       G_PRIORITY_DEFAULT,
+                                       cancellable);
+  g_object_unref (simple);
+}
+
+
+/**
+ * zpj_skydrive_download_file_id_to_stream_finish:
+ * @self: A #ZpjSkydrive.
+ * @res: A #GAsyncResult.
+ * @error: (allow-none): An optional #GError, or %NULL.
+ *
+ * Finishes an asynchronous operation started with
+ * zpj_skydrive_download_file_id_to_stream_async().
+ *
+ * Returns: (transfer full): A #GInputStream to read the file data
+ * from. Free the returned object with g_object_unref().
+ */
+GInputStream *
+zpj_skydrive_download_file_id_to_stream_finish (ZpjSkydrive *self,
+                                                GAsyncResult *res,
+                                                GError **error)
+{
+  GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (res);
+  GInputStream *ret_val = NULL;
+  ZpjSkydriveThreadData *data;
+
+  g_return_val_if_fail (g_simple_async_result_is_valid (res,
+                                                        G_OBJECT (self),
+                                                        zpj_skydrive_download_file_id_to_stream_async),
+                        NULL);
+  g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+  if (g_simple_async_result_propagate_error (simple, error))
+    goto out;
+
+  data = (ZpjSkydriveThreadData *) g_simple_async_result_get_op_res_gpointer (simple);
+  ret_val = G_INPUT_STREAM (g_value_get_object (&data->result));
+
+ out:
+  return ret_val;
+}
+
+
 /**
  * zpj_skydrive_download_file_to_stream:
  * @self: A #ZpjSkydrive.
@@ -480,7 +591,8 @@ zpj_skydrive_download_file_id_to_stream (ZpjSkydrive *self,
  *
  * Synchronously returns a stream for downloading @file from
  * <ulink url="http://msdn.microsoft.com/en-us/library/live/hh826521">
- * Skydrive</ulink>.
+ * Skydrive</ulink>. See zpj_skydrive_download_file_to_stream_async()
+ * for the asynchronous version of this call.
  *
  * Returns: (transfer full): A #GInputStream to read the file data
  * from. Free the returned object with g_object_unref().
@@ -504,6 +616,63 @@ zpj_skydrive_download_file_to_stream (ZpjSkydrive *self,
 }
 
 
+/**
+ * zpj_skydrive_download_file_to_stream_async:
+ * @self: A #ZpjSkydrive.
+ * @file: The #ZpjSkydriveFile to be downloaded.
+ * @cancellable: (allow-none): An optional #GCancellable object, or
+ *   %NULL.
+ * @callback: (scope async): A #GAsyncReadyCallback to call when the
+ *   request is satisfied.
+ * @user_data: (closure): The data to pass to @callback.
+ *
+ * Asynchronously returns a stream for downloading @file from
+ * <ulink url="http://msdn.microsoft.com/en-us/library/live/hh826521">
+ * Skydrive</ulink>. See zpj_skydrive_download_file_to_stream() for the
+ * synchronous version of this call.
+ *
+ * When the operation is finished, @callback will be called. You can
+ * then call zpj_skydrive_download_file_to_stream_finish() to get the
+ * result of the operation.
+ */
+void
+zpj_skydrive_download_file_to_stream_async (ZpjSkydrive *self,
+                                            ZpjSkydriveFile *file,
+                                            GCancellable *cancellable,
+                                            GAsyncReadyCallback callback,
+                                            gpointer user_data)
+{
+  const gchar *file_id;
+
+  g_return_if_fail (ZPJ_IS_SKYDRIVE (self));
+  g_return_if_fail (ZPJ_IS_SKYDRIVE_FILE (file));
+
+  file_id = zpj_skydrive_entry_get_id (ZPJ_SKYDRIVE_ENTRY (file));
+  g_return_if_fail (file_id != NULL && file_id[0] != '\0');
+
+  zpj_skydrive_download_file_id_to_stream_async (self, file_id, cancellable, callback, user_data);
+}
+
+
+/**
+ * zpj_skydrive_download_file_to_stream_finish:
+ * @self: A #ZpjSkydrive.
+ * @res: A #GAsyncResult.
+ * @error: (allow-none): An optional #GError, or %NULL.
+ *
+ * Finishes an asynchronous operation started with
+ * zpj_skydrive_download_file_to_stream_async().
+ *
+ * Returns: (transfer full): A #GInputStream to read the file data
+ * from. Free the returned object with g_object_unref().
+ */
+GInputStream *
+zpj_skydrive_download_file_to_stream_finish (ZpjSkydrive *self, GAsyncResult *res, GError **error)
+{
+  return zpj_skydrive_download_file_id_to_stream_finish (self, res, error);
+}
+
+
 /**
  * zpj_skydrive_download_file_id_to_path:
  * @self: A #ZpjSkydrive.
